﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;
using C1.Silverlight.FlexGrid;
using System.ComponentModel.DataAnnotations;
using C1.Silverlight;

namespace FlexGridDragDropSL
{
    public partial class MainPage : UserControl
    {
        ObservableCollection<Customer> flex1source;
        ObservableCollection<Customer> flex2source;
        public MainPage()
        {
            InitializeComponent();

            flex1source = Customer.GetCustomerList(20);
            flex2source = Customer.GetCustomerList(20);
            flex1.ItemsSource = flex1source;
            flex2.ItemsSource = flex2source;

            var ddm = new C1DragDropManager();
            flex1.CellFactory = new DragDropCellFactory(ddm);
            flex2.CellFactory = new DragDropCellFactory(ddm);



            ddm.RegisterDropTarget(flex1, true);
            ddm.RegisterDropTarget(flex2, true);
        
            ddm.DragStart += (s, e) =>
                {
                    var bdr = e.DragSource as Border;
                    var row = (bdr.Tag as Row);
                    var flex = (bdr.Tag as Row).Grid;
                     ddm.TargetMarker.Width = flex.ActualWidth;  //configure the Target Marker
         
                };

            ddm.DragDrop += (s, e) =>
                {
                    var bdr = e.DragSource as Border;
                    var flex = e.DropTarget as C1FlexGrid;
                    var row = flex.HitTest(e.GetPosition(flex)).Row; 

                    if (flex.Name == "flex1")  //check the actual drop target
                    {
                        flex1source.Insert(row , (bdr.Tag as Row).DataItem as Customer);
                        flex2source.RemoveAt((bdr.Tag as Row).Index);
                    }
                    else if (flex.Name == "flex2")
                    {
                        flex2source.Insert(row , (bdr.Tag as Row).DataItem as Customer);
                        flex1source.RemoveAt((bdr.Tag as Row).Index);
                    }

                };
        }
    }

    public class DragDropCellFactory : CellFactory
    {
        private C1DragDropManager _ddm;
        public DragDropCellFactory(C1DragDropManager ddm)
        {
            _ddm = ddm;
        }
        public override void CreateCellContent(C1FlexGrid grid, Border bdr, CellRange rng)
        {
            base.CreateCellContent(grid, bdr, rng);
            bdr.Tag = grid.Rows[rng.Row]; //used to find the actual row dragged
            _ddm.RegisterDragSource(bdr, DragDropEffect.Move, ModifierKeys.None);
        }
    }
    
    public class Customer :
       INotifyPropertyChanged,
       IEditableObject
    {
        // ** fields
        int _id, _countryID;
        string _first, _last;
    
        // ** data generators
        static Random _rnd = new Random();
        static string[] _firstNames = "Andy|Ben|Charlie|Dan|Ed|Fred|Gil|Herb|Jack|Karl|Larry|Mark|Noah|Oprah|Paul|Quince|Rich|Steve|Ted|Ulrich|Vic|Xavier|Zeb".Split('|');
        static string[] _lastNames = "Ambers|Bishop|Cole|Danson|Evers|Frommer|Griswold|Heath|Jammers|Krause|Lehman|Myers|Neiman|Orsted|Paulson|Quaid|Richards|Stevens|Trask|Ulam".Split('|');
        static string[] _countries = "China|India|United States|Indonesia|Brazil|Pakistan|Bangladesh|Nigeria|Russia|Japan|Mexico|Philippines|Vietnam|Germany|Ethiopia|Egypt|Iran|Turkey|Congo|France|Thailand|United Kingdom|Italy|Myanmar".Split('|');

        // ** ctors
        public Customer()
            : this(_rnd.Next())
        {
        }
        public Customer(int id)
        {
            ID = id;
            
            First = GetString(_firstNames);
            Last = GetString(_lastNames);
            CountryID = _rnd.Next() % _countries.Length;         
        }

        // ** object model
        public int ID
        {
            get { return _id; }
            set
            {
                if (value != _id)
                {
                    _id = value;
                    RaisePropertyChanged("ID");
                }
            }
        }
        
        //private string Name
        //{
        //    get { return string.Format("{0} {1}", First, Last); }
        //}
        public string Country
        {
            get { return _countries[_countryID]; }
        }
        public int CountryID
        {
            get { return _countryID; }
            set
            {
                if (value != _countryID && value > -1 && value < _countries.Length)
                {
                    _countryID = value;

                    // call OnPropertyChanged with null parameter since setting this property
                    // modifies the value of "CountryID" and also the value of "Country".
                    RaisePropertyChanged(null);
                }
            }
        }

     
        public string First
        {
            get { return _first; }
            set
            {
                if (value != _first)
                {
                    _first = value;

                    // call OnPropertyChanged with null parameter since setting this property
                    // modifies the value of "First" and also the value of "Name".
                    RaisePropertyChanged(null);
                }
            }
        }
      
        public string Last
        {
            get { return _last; }
            set
            {
                if (value != _last)
                {
                    _last = value;

                    // call OnPropertyChanged with null parameter since setting this property
                    // modifies the value of "First" and also the value of "Name".
                    RaisePropertyChanged(null);
                }
            }
        }
       

        // some read-only stuff
      

        // ** utilities
        static string GetString(string[] arr)
        {
            return arr[_rnd.Next(arr.Length)];
        }
        static string GetName()
        {
            return string.Format("{0} {1}", GetString(_firstNames), GetString(_lastNames));
        }

        // ** static list provider
        public static ObservableCollection<Customer> GetCustomerList(int count)
        {
            var list = new ObservableCollection<Customer>();
            for (int i = 0; i < count; i++)
            {
                list.Add(new Customer(i));
            }
            return list;
        }

        // ** static value providers
        public static string[] GetCountries() { return _countries; }
        public static string[] GetFirstNames() { return _firstNames; }
        public static string[] GetLastNames() { return _lastNames; }

        //-----------------------------------------------------------------------------------
        #region ** INotifyPropertyChanged Members

        // this interface allows bounds controls to react to changes in the data objects.

        void RaisePropertyChanged(string propertyName)
        {
            OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }
        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, e);
        }

        #endregion

        //-----------------------------------------------------------------------------------
        #region IEditableObject Members

        // this interface allows transacted edits (user can press escape to restore previous values).

        Customer _clone;
        public void BeginEdit()
        {
            _clone = (Customer)this.MemberwiseClone();
        }
        public void EndEdit()
        {
            _clone = null;
        }
        public void CancelEdit()
        {
            if (_clone != null)
            {
                foreach (var p in this.GetType().GetProperties())
                {
                    if (p.CanRead && p.CanWrite)
                    {
                        p.SetValue(this, p.GetValue(_clone, null), null);
                    }
                }
            }
        }
        #endregion
    }

}
